SystemOrganization addCategory: #'NXTalk-Kernel'!
SystemOrganization addCategory: #'NXTalk-Building'!
SystemOrganization addCategory: #'NXTalk-Collections'!
SystemOrganization addCategory: #'NXTalk-Tests'!

!String methodsFor: '*nxtalk' stamp: 'mab 3/11/2008 01:09'!
printNl
	Transcript show: self; cr.! !

RBProgramNodeVisitor subclass: #NXTCompiler
	instanceVariableNames: 'image literals bytecodes primitive half arguments temporaries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTCompiler class methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:20'!
bytecodes
	^ Dictionary new
		at: #return put: 0;
		at: #send put: 1;
		at: #selfSend put: 2;
		at: #push put: 3;
		at: #popEmpty put: 4;
		at: #returnSelf put: 5;
		at: #pop put: 6;
		at: #dup put: 7;
		yourself! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:37'!
acceptAssignmentNode: anAssignmentNode
	| pushNeeded |
	self visitNode: anAssignmentNode value.
	pushNeeded := anAssignmentNode parent isSequence not.
	pushNeeded ifTrue: [ self addBytecode: #dup ].
	self addBytecode: #pop.
	self addVariable: anAssignmentNode variable name.
	^ pushNeeded! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:48'!
acceptCascadeNode: cascadeNode
	^ true! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:33'!
acceptLiteralNode: aLiteralNode
	| string |
	self addBytecode: #push.
	string := NXTImageSymbol new symbol: aLiteralNode value.
	self image addObject: string.
	self addLiteral: string.
	^ true! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:33'!
acceptMessageNode: aMessageNode
	self primitive ifNotNil: [ ^ self ].
	aMessageNode arguments do: [:each | self visitNode: each].
	(aMessageNode receiver isVariable and: [ aMessageNode receiver name = 'self']) 
		ifTrue: [ self addBytecode: #selfSend ]
		ifFalse: [
			(aMessageNode isCascaded not or: [aMessageNode isFirstCascaded]) 
			ifTrue: [self visitNode: aMessageNode receiver].
			self addBytecode: #send.
		].
	self addLiteral: (self image symbolAt: aMessageNode selector).
	^ true! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:34'!
acceptPragmaNode: aPragmaNode
	aPragmaNode pragma keyword = #NXTPrimitive: ifTrue: [
		self primitive: (aPragmaNode pragma argumentAt: 1) ].! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:34'!
acceptReturnNode: aReturnNode
	super acceptReturnNode: aReturnNode.
	self addBytecode: #return.
	^ false! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:37'!
acceptSequenceNode: aSequenceNode 
	self visitTemporaryVariables: aSequenceNode temporaries.
	aSequenceNode statements do: [:each | (self visitNode: each) ifTrue: [self addBytecode: #popEmpty]].
	half ifTrue: [ self addBytecode: #returnSelf ].! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:35'!
acceptVariableNode: variableNode
	| index |
	index := self arguments indexOf: variableNode name.
	index isZero ifFalse: [ self addBytecode: #push.
		self addArgument: index - 1 ].
	index := self temporaries indexOf: variableNode name.
	index isZero ifFalse: [ self addBytecode: #push.
		self addTemporary: index - 1].
	variableNode name = 'true' ifTrue: [
		self addBytecode: #push.
		self addLiteral: (self image specialObjectAt: #true) ].
	^ true! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/22/2008 14:28'!
addArgument: index
	self addBytecodeArgument: (index bitAnd: 16r1F)! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 11:06'!
addBytecode: aSymbol
	| bytecode |
	bytecode := self class bytecodes at: aSymbol.
	"Bytecodes only have 4 bits, thus they get packed"
	half 
		ifFalse: [ self bytecodes add: bytecode << 4. half := true ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: bytecode). half := false ].! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:06'!
addBytecodeArgument: aNumber
	"Arguments always have 8 bits, thus they could be split over byte boundaries"
	half 
		ifFalse: [ self bytecodes add: aNumber ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: aNumber >> 4).
			self bytecodes add: (aNumber bitAnd: 16rF) << 4 ].! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/22/2008 14:27'!
addLiteral: literal
	self addBytecodeArgument: (16r80 bitOr: (self mapLiteral: literal))! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 15:06'!
addTemporary: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 2 << 5)! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 14:50'!
addVariable: aName
	self ifArgument: aName doWithIndex: [ :index |
		self addArgument: index ].
	self ifTemporary: aName doWithIndex: [ :index |
		self addTemporary: index ].
	! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/21/2008 18:34'!
arguments
	^ arguments! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/21/2008 18:34'!
arguments: anObject
	arguments := anObject! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/15/2008 17:46'!
bytecodes
	^ bytecodes! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/15/2008 17:46'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 16:05'!
compileMethod: aCompiledMethod
	| ast |
	ast := SqueakParser parseMethod: aCompiledMethod getSource.
	self visitNode: ast.
	self primitive ifNil: [ ^ (NXTImageCompiledMethod 
		bytecodes: self bytecodes 
		literals: self literals)
		tempCount: self temporaries size ].
	^ NXTImageCompiledMethod primitive: self primitive! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 14:49'!
ifArgument: aName doWithIndex: aBlock
	| index |
	index := self arguments indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 14:50'!
ifTemporary: aName doWithIndex: aBlock
	| index |
	index := self temporaries indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:14'!
image
	^ image! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:14'!
image: anObject
	image := anObject! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 14:51'!
initialize
	literals := OrderedCollection new.
	bytecodes := OrderedCollection new.
	arguments := OrderedCollection new.
	temporaries := OrderedCollection new.
	half := false.! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/15/2008 17:49'!
literals
	^ literals! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/15/2008 17:49'!
literals: anObject
	literals := anObject! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:23'!
mapLiteral: aLiteral
	^ (self literals indexOf: aLiteral ifAbsent: [ self literals add: aLiteral. self literals size ]) - 1! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:37'!
primitive
	^ primitive! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:08'!
primitive: anObject
	primitive := anObject! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 14:03'!
temporaries
	^ temporaries! !

!NXTCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 14:03'!
temporaries: anObject
	temporaries := anObject! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/21/2008 18:34'!
visitArgument: argumentNode
	self arguments add: argumentNode name! !

!NXTCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 14:29'!
visitTemporaryVariable: aNode
	self temporaries add: aNode name.! !

Object subclass: #NXTImage
	instanceVariableNames: 'objects specialObjects symbols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/25/2008 23:50'!
addObject: anNXTObject
	^  self objects add: anNXTObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:12'!
initialize
	objects := OrderedCollection new.
	specialObjects := Dictionary new.
	symbols := Dictionary new.! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/25/2008 23:38'!
metaclass
	^ self specialObjects at: #metaclass! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/25/2008 23:38'!
metaclass: aValue
	^ self specialObjects at: #metaclass put: aValue! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/20/2008 03:18'!
nil
	^ self specialObjects at: #nil! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/20/2008 03:18'!
nil: aValue
	^ self specialObjects at: #nil put: aValue! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects
	^ objects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects: anObject
	objects := anObject! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects
	^ specialObjects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects: anObject
	specialObjects := anObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:24'!
symbolAt: aSymbol
	^ symbols at: aSymbol ifAbsentPut: [ self addObject: (NXTImageSymbol new symbol: aSymbol) ]! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/25/2008 23:25'!
systemDictionary
	^ self specialObjects at: #systemDictionary! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 2/25/2008 23:25'!
systemDictionary: aValue
	^ self specialObjects at: #systemDictionary put: aValue! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/12/2008 14:25'!
writeHeaderTo: aByteStream
	aByteStream littleEndianWord: 16r4242.
	aByteStream littleEndianWord: self objects size.! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 3/17/2008 17:12'!
writeObjectsTo: aByteStream withMapper: mapper
	self objects do: [ :each | each writeOn: aByteStream withMapper: mapper ].! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/17/2008 15:38'!
writeSpecialObjectsTo: aByteStream withMapper: mapper
	aByteStream 
		littleEndianWord: (mapper idFor: (self specialObjects at: #System));
		littleEndianWord: (mapper idFor: (self specialObjects at: #runSymbol))..! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 3/17/2008 17:13'!
writeToStream: aByteStream
	| mapper |
	mapper := NXTObjectMapper on: self objects.
	self writeHeaderTo: aByteStream.
	self writeObjectsTo: aByteStream withMapper: mapper.
	self writeSpecialObjectsTo: aByteStream withMapper: mapper.! !

Object subclass: #NXTImageBuilder
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageBuilder class methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 02:52'!
standardCategories
	^ #('NXTalk-Kernel' 'NXTalk-Collections')! !

!NXTImageBuilder class methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:06'!
systemClasses
	^ #(objectClass nilClass classClass arrayClass symbolClass methodClass integerClass bigintegerClass frameClass stringClass doubleClass)! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 4/17/2008 18:04'!
compileClass: aClass
	| classObject method methodDictionary |
	classObject := NXTImageObject new.
	self image addObject: classObject.
	methodDictionary := NXTImageObject new.
	self image addObject: methodDictionary.
	classObject instanceVariables add: methodDictionary.
	aClass selectors do: [ :selector |
		method := NXTImageCompiledMethod 
			from: (aClass compiledMethodAt: selector) 
			within: self image.
		self image addObject: method.
		methodDictionary instanceVariables add: (self image symbolAt: selector).
		methodDictionary instanceVariables add: method.
	].
	^ classObject! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 03:00'!
compileClasses
	self class standardCategories do: [ :category |
		(Smalltalk organization listAtCategoryNamed: category) do: [ :class |
			self compileClass: class ]].! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:53'!
createMetaclasses
	| metaclass |
	metaclass := self newClass.
	metaclass nxtClass: self newClass.
	"Metaclass class class = Metaclass"
	metaclass nxtClass nxtClass: metaclass.
	^ metaclass! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:24'!
createSpecialObjects
	self image nil: self newObject.
	self image metaclass: self createMetaclasses.
	self createSystemClasses.
	self image nil nxtClass: (self image specialObjects at: #nilClass).
	self initializeSystemClasses.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 03:06'!
createStandardImage
	self image: NXTImage new.
	self createSpecialObjects.
	self compileClasses.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:06'!
createSystemClasses
	self class systemClasses do: [ :symbol |
		self image specialObjects at: symbol put: self newSystemClass ].
	! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 4/17/2008 18:10'!
helloWorldImageOn: aByteStream
	| systemClass systemObject |
	self image: NXTImage new.
	systemClass := self compileClass: NXTSystem.
	systemObject := NXTImageObject new.
	self image addObject: systemObject.
	self image specialObjects at: #System put: systemObject.
	systemObject nxtClass: systemClass.
	systemClass nxtClass: systemClass.
	self image specialObjects at: #runSymbol put: (self image symbolAt: #run).
	self image writeToStream: aByteStream.
! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image
	^ image! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image: anObject
	image := anObject! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 00:39'!
initializeSystemClass: classSymbol superclass: superclassSymbol name: nameSymbol
	| class |
	class := self image specialObjects at: classSymbol.
	class nxtSuperclass: (self image specialObjects at: superclassSymbol).
	(self image specialObjects at: #Smalltalk) at: nameSymbol put: class.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/26/2008 01:00'!
initializeSystemClasses
	self initializeSystemClass: #objectClass superclass: #nil name: #Object.
	self initializeSystemClass: #classClass superclass: #objectClass name: #Class.
	self initializeSystemClass: #metaclass superclass: #classClass name: #Metaclass.
	self initializeSystemClass: #nilClass superclass: #objectClass name: #UndefinedObject.
	self initializeSystemClass: #arrayClass superclass: #objectClass name: #Array.
	self initializeSystemClass: #methodClass superclass: #arrayClass name: #Method.
	self initializeSystemClass: #symbolClass superclass: #objectClass name: #Symbol.! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:52'!
newClass
	^  self image addObject: NXTImageClass new! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:52'!
newObject
	^  self image addObject: NXTImageObject new! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:56'!
newSystemClass
	| systemClass |
	systemClass := self newClass.
	systemClass nxtClass: self newClass.
	systemClass nxtClass nxtClass: self image metaclass.
	^ systemClass! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:36'!
objectForClass: anNXTClass
	| class |
	class := NXTImageClass new.
	class nxtClass: self newMetaclass.
	
	^ class! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:17'!
specialObjects
	^ self image specialObjects! !

Object subclass: #NXTImageObject
	instanceVariableNames: 'nxtClass instanceVariables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

NXTImageObject subclass: #NXTImageClass
	instanceVariableNames: 'methodDictionary instanceVariableNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageClass methodsFor: 'accessing' stamp: 'mab 4/17/2008 13:51'!
methodDictionary
	^ methodDictionary! !

!NXTImageClass methodsFor: 'accessing' stamp: 'mab 4/17/2008 13:51'!
methodDictionary: anObject
	methodDictionary := anObject! !

NXTImageObject subclass: #NXTImageCompiledMethod
	instanceVariableNames: 'literals bytecodes primitive tempCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/22/2008 14:14'!
bytecodes: bytecodes literals: literals
	^self new 
		bytecodes: bytecodes;
		literals: literals! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:17'!
from: aCompiledMethod within: anNXTImage
	^ NXTCompiler new
		image: anNXTImage;
		compileMethod: aCompiledMethod! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:41'!
primitive: aNumber
	^ self new primitive: aNumber! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes
	^ bytecodes! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:20'!
computeSize
	^ self primitive
		ifNil: [ 2 + self bytecodes size + (2 * self literals size) ]
		ifNotNil: [ 2 ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 17:24'!
elementSize
	^ 1! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals
	^ literals! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals: anObject
	literals := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
methodHeader
	^ self primitive 
		ifNil: [ (self literals size bitAnd: 16r7F) << 8 bitOr: self tempCount << 3 ]
		ifNotNil: [ 1 << 15 bitOr: (self primitive bitAnd: 16r7FFF) ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive
	^ primitive! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive: anObject
	primitive := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount
	^ tempCount! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount: anObject
	tempCount := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:26'!
writeDataOn: aByteStream withMapper: mapper
	self writeMethodHeaderOn: aByteStream.
	self primitive ifNil: [
		self bytecodes do: [ :bytecode | aByteStream nextPut: bytecode ].
		self literals do: [ :literal | aByteStream littleEndianWord: (mapper idFor: literal) ].
	].! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
writeMethodHeaderOn: aByteStream
	aByteStream littleEndianWord: self methodHeader.! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:42'!
computeSize
	^ self instanceVariables size! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:44'!
elementSize
	^ 2! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:19'!
header
	| header |
	"TODO this code assumes, size is below 2048"
	header := ((self elementSize bitAnd: 3) << 12 bitOr: 1 << 11) bitOr: (self computeSize bitAnd: 16r7FF).
	^ header! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 2/25/2008 23:05'!
initialize
	instanceVariables := OrderedCollection new.! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables
	^ instanceVariables! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables: anObject
	instanceVariables := anObject! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:04'!
nxtClass
	^ nxtClass! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:04'!
nxtClass: anObject
	nxtClass := anObject! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 16:33'!
writeDataOn: aByteStream withMapper: mapper
	self instanceVariables do: [ :instanceVariable |
		aByteStream littleEndianWord: (mapper idFor: instanceVariable) ].! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:39'!
writeHeaderOn: aByteStream
	aByteStream littleEndianWord: self header.! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:51'!
writeOn: aByteStream withMapper: mapper
	self writeHeaderOn: aByteStream.
	aByteStream littleEndianWord: (mapper idFor: self nxtClass).
	self writeDataOn: aByteStream withMapper: mapper.! !

NXTImageObject subclass: #NXTImageSymbol
	instanceVariableNames: 'symbol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:06'!
computeSize
	^ self symbol size! !

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:06'!
elementSize
	^ 1! !

!NXTImageSymbol methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:09'!
symbol
	^ symbol! !

!NXTImageSymbol methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:09'!
symbol: anObject
	symbol := anObject! !

!NXTImageSymbol methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 18:08'!
writeDataOn: aByteStream withMapper: mapper
	aByteStream nextPutAll: self symbol! !

Object subclass: #NXTObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTArray subclass: #NXTCompiledMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTBoolean subclass: #NXTFalse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTBoolean subclass: #NXTTrue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Collections'!

NXTObject subclass: #NXTMetaclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 3/11/2008 01:09'!
printNl
	<primitive: 500>! !

NXTString subclass: #NXTSymbol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

NXTObject subclass: #NXTSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/21/2008 17:17'!
hateWorldMessage
	^ 'Hate you, world!!'! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/21/2008 16:47'!
helloWorldMessage
	^ 'Hello World'! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/20/2008 20:12'!
primDrive
	<NXTPrimitive: 1>! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/21/2008 15:34'!
printString: aString
	<NXTPrimitive: 0>
	Transcript show: aString; cr.! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 16:31'!
run
	| temp1 temp2 |
	temp1 := temp2 := self helloWorldMessage.
	self sayHello: temp1.
	self sayHello: temp2.
! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 4/21/2008 18:28'!
sayHello: message
	self printString: message! !

NXTObject subclass: #NXTUndefinedObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Kernel'!

Object subclass: #NXTObjectMapper
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTObjectMapper class methodsFor: 'as yet unclassified' stamp: 'mab 3/17/2008 17:15'!
on: objectList
	^ self new objects: objectList! !

!NXTObjectMapper methodsFor: 'as yet unclassified' stamp: 'mab 4/17/2008 14:30'!
idFor: anObject
	anObject 
		ifNil: [ ^ 0 ]
		ifNotNil: [ ^ (self objects indexOf: anObject) - 1 ]! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects
	^ objects! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects: anObject
	objects := anObject! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord
	self nextLittleEndianNumber: 2! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord: number
	self nextLittleEndianNumber: 2 put: number! !

TestCase subclass: #NXTObjectMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Tests'!
